#!/usr/bin/env python

import json
import os
import unittest

import httplib2
import yaml

import amulet


opd = os.path.dirname

root_dir = opd(opd(opd((os.path.abspath(__file__)))))


class AmuletDeployment(object):
    """Fixture for a deployed set of juju services from a deployer config"""

    def __init__(self, deployer_cfg, juju_env=None, sentries=False):
        self.deployment = amulet.Deployment(juju_env, sentries=sentries)
        with open(deployer_cfg) as f:
            jd_script = yaml.safe_load(f.read())
        # Use juju_env given or defaulted above rather than taking the name
        # from the deployer config and assuming an env of that name exists
        script = {self.deployment.juju_env: jd_script.values()[0]}
        self.deployment.load(script)

    def setUp(self):
        self.deployment.setup()


class PPAAssignerTest(unittest.TestCase):
    """Integration tests for ppa-assigner service run on a juju deployment"""

    def setUp(self):
        super(PPAAssignerTest, self).setUp()
        self.deployer = AmuletDeployment(os.path.join(root_dir,
                                                      'juju-deployer',
                                                      'ppa-assigner.yaml'))
        self.deployer.setUp()

    def get_ip_and_port(self):
        self.status = amulet.waiter.status(self.deployer.deployment.juju_env)
        units = self.status['services']['django']['units']
        ip = units['django/0']['public-address']
        port, _ = units['django/0']['open-ports'][0].split('/')
        return ip, port

    def get_server_status_and_content(self, url):
        final_url = url.format(*self.get_ip_and_port())

        # Issue a simple command against the ppa assigner API
        client = httplib2.Http()
        resp, content = client.request(final_url, 'GET')
        jcontent = json.loads(content)
        return resp, jcontent

    def test_ppa_api(self):
        url = 'http://{}:{}/api/v1/ppa'
        resp, content = self.get_server_status_and_content(url)

        self.assertEqual('200', resp['status'])
        self.assertNotEqual([], content['objects'])

    def test_ppa_name(self):
        url = 'http://{}:{}/api/v1/ppa'
        resp, content = self.get_server_status_and_content(url)
        self.assertIn('ppa:ci-engineering-airline/ci-pool-',
                      content['objects'][0]['name'])

    def test_next_page_ppa(self):
        url = 'http://{}:{}/api/v1/ppa/?format=json&limit=1'
        resp, content = self.get_server_status_and_content(url)

        self.assertEqual('/api/v1/ppa/?offset=1&limit=1&format=json',
                         content['meta']['next'])
        self.assertEqual('/api/v1/ppa/1/',
                         content['objects'][0]['resource_uri'])

    def test_previous_page_ppa(self):
        url = 'http://{}:{}/api/v1/ppa/?format=json&limit=1&offset=1'
        resp, content = self.get_server_status_and_content(url)

        self.assertEqual('/api/v1/ppa/?format=json&limit=1&offset=0',
                         content['meta']['previous'])
        self.assertEqual('/api/v1/ppa/2/',
                         content['objects'][0]['resource_uri'])

if __name__ == "__main__":
    unittest.main()
